//
// Copyright (c) 2001-2014 by PDFTron Systems Inc. All Rights Reserved.
//

using System;
using pdftron;
using pdftron.Common;
using pdftron.Filters;
using pdftron.SDF;
using pdftron.PDF;
using pdftron.PDF.OCG;
using System.Collections.Generic;

/// <summary>
//-----------------------------------------------------------------------------------
// This sample demonstrates how to create layers in PDF.
// The sample also shows how to extract and render PDF layers in documents 
// that contain optional content groups (OCGs)
//
// With the introduction of PDF version 1.5 came the concept of Layers. 
// Layers, or as they are more formally known Optional Content Groups (OCGs),
// refer to sections of content in a PDF document that can be selectively 
// viewed or hidden by document authors or consumers. This capability is useful 
// in CAD drawings, layered artwork, maps, multi-language documents etc.
//
// Notes: 
// ---------------------------------------
// - This sample is using CreateLayer() utility method to create new OCGs. 
//   CreateLayer() is relatively basic, however it can be extended to set 
//   other optional entries in the 'OCG' and 'OCProperties' dictionary. For 
//   a complete listing of possible entries in OC dictionary please refer to 
//   section 4.10 'Optional Content' in the PDF Reference Manual.
// - The sample is grouping all layer content into separate Form XObjects. 
//   Although using PDFNet is is also possible to specify Optional Content in 
//   Content Streams (Section 4.10.2 in PDF Reference), Optional Content in  
//   XObjects results in PDFs that are cleaner, less-error prone, and faster 
//   to process.
//-----------------------------------------------------------------------------------
/// </summary>
namespace PDFLayersTestCS
{
	class Class1 
	{			
		private static pdftron.PDFNetLoader pdfNetLoader = pdftron.PDFNetLoader.Instance();

		[STAThread]
		static void Main(string[] args)
		{
			PDFNet.Initialize();

			try
			{
				using (PDFDoc doc = new PDFDoc(args[0]))
				{
                    // Create OCG Layer.
                    // second parameter is what user should see in PDF viewer UI
                    Group annotation_layer = CreateLayer(doc, "Annotation Layer");

                    PageIterator itr = doc.GetPageIterator();
                    for (; itr.HasNext(); itr.Next())
                    {
                        Page page = itr.Current();
                        Obj res_dict = page.GetResourceDict();
                        if(res_dict == null) res_dict = page.GetSDFObj().PutDict("Resources");
                        Obj xobjects = res_dict.FindObj("XObject");
                        if(xobjects == null) xobjects = res_dict.PutDict("XObject");

                        // Iterate through all Form XObjects associated with this page.
                        HashSet<string> existingFormXobjects = new HashSet<string>();
                        for (DictIterator ditr = xobjects.GetDictIterator(); ditr.HasNext(); ditr.Next())
                        {
                            existingFormXobjects.Add(ditr.Key().GetName());
                        }

                        for (int i = 0; i < page.GetNumAnnots();)
                        {
                            Annot annot = page.GetAnnot(i);
                            if (annot.IsValid() == false)
                            {
                                page.AnnotRemove(i);
                            }
                            else
                            { 
                                // Flatten removes the annotation from the page
                                // Creates a Form XObject of the annotation
                                // And adds the Form Xobject to the page content stream
                                annot.Flatten(page);
                            }
                        }

                        // Now we just run through all the new Xobjects, and add them to the OCG layer
                        for (DictIterator ditr = xobjects.GetDictIterator(); ditr.HasNext(); ditr.Next())
                        {
                            if (!existingFormXobjects.Contains(ditr.Key().GetName()))
                            {
                                Obj xobject = ditr.Value();
                                xobject.Put("OC", annotation_layer.GetSDFObj());
                            }
                        }
                    }

					doc.Save(args[1], SDFDoc.SaveOptions.e_linearized);
					Console.WriteLine("Done.");

				}
			}
			catch (PDFNetException e)
			{
				Console.WriteLine(e.Message);
			}
		}

		// A utility function used to add new Content Groups (Layers) to the document.
		static Group CreateLayer(PDFDoc doc, String layer_name)
		{
			Group grp = Group.Create(doc, layer_name);
			Config cfg = doc.GetOCGConfig();
			if (cfg == null) 
			{
				cfg = Config.Create(doc, true);
				cfg.SetName("Default");
			}

			// Add the new OCG to the list of layers that should appear in PDF viewer GUI.
			Obj layer_order_array = cfg.GetOrder();
			if (layer_order_array == null) 
			{
				layer_order_array = doc.CreateIndirectArray();
				cfg.SetOrder(layer_order_array);
			}
			layer_order_array.PushBack(grp.GetSDFObj());

			return grp;
		}
	}
}
